#!/usr/bin/python
import sys
import pdb
import pickle
import random
import optparse

def mt(i, j, p):
  if i < j:
    return (i,j), p + 1
  else:
    return (j,i), -p + 1

def chooseK(alist, k):
  # returns a list of k distinct elements selected from alist at random. 
  # alist is not changed
  assert k <= len(alist)
  res = []
  for i in range(0,k):
    while True:
      el = random.choice(alist)
      if el not in res:
        res.append(el)
        break
  return res


# parse arguments
argv = sys.argv
parser = optparse.OptionParser()
parser.add_option("-t", "--top-file", action="store",
                  type="string", dest="top",
                  help="specify the input topology file [required]")
parser.add_option("-o", "--output-file", action="store",
                  type="string", dest="out",
                  help="specify the output file [required]")
parser.add_option("-p", "--base-port", action="store",
                  type="int", dest="port", default=-1, 
                  help="specify the base port to compute router listening"
                  "ports. Router i listens of port+10*i [required]")
parser.add_option("-c", "--ctrl-types", action="store",
                  type="int", dest="ctrl", default=0, 
                  help="specify which controllers should be used: -3-lt,"
                  "-2-pv,-1-half/half, positive number is the exact number "
                  "of LT nodes that should be choosed randomly")



(opts, args) = parser.parse_args(argv) 

if len(args) > 1:
  print "Everyting is specified using options"
  parser.print_help()
  sys.exit(2)
if opts.top == None or opts.out == None or opts.port == -1:
  print "Please specify all required options"
  parser.print_help()
  sys.exit(2)

fin = open(opts.top, "r")
fout = open(opts.out, "w")
rels = {} # {edge -> policy}
rtrs = set() # set(asid)

# read the input topology file
for line in fin:
  ps = line.split()
  if len(ps) == 0 or ps[0] == "#":
    continue
  assert len(ps) == 3

  a, b, p = map(lambda x: int(x), ps)
  e, p = mt(a,b,p)
  rtrs.add(a)
  rtrs.add(b)
  if e not in rels:
    rels[e] = p
  else:
    assert rels[e] == p

if opts.ctrl > 0:
  # random seed values 58367, 2196, 80049, 27778, 750338
  #random.seed(58367)
  #random.seed(2196)
  #random.seed(80049)
  #random.seed(27778)
  #random.seed(750338)
  random.seed(int(10000*random.random()))
  random.seed(int(10000*random.random()))
  random.seed(int(10000*random.random()))
  lt_rtrs = chooseK(list(rtrs), opts.ctrl)
  print lt_rtrs

# write the output file
fout.write("section physical_topology none\n")
for e in rels.iterkeys():
  fout.write("%d %d %d %d\n" % (e[0], e[0], e[1], e[1]))
fout.write("section nls PLNlsOrpa\n")
for r in rtrs:
  fout.write("%d %d localhost %d\n" % (r, r, opts.port+10*r))
fout.write("section irp PLIrpOrpa\n")
fout.write("section vman PLVNodeManagerOrpa\n")
for r in rtrs:
  fout.write("%d %d\n" % (r, r))
fout.write("section cc PLControllerConnectorOrpa\n")
fout.write("section ctrl PLControllerLTOrpa\n")
fout.write("subsection controller_instances\n")
for r in rtrs:
  if opts.ctrl == -3:
    fout.write("%d %d PLControllerLTOrpa\n" % (r, r))
  elif opts.ctrl == -2:
    fout.write("%d %d PLControllerPVOrpa\n" % (r, r))
  elif opts.ctrl == -1:
    if r % 2 == 0:
      fout.write("%d %d PLControllerLTOrpa\n" % (r, r))
    else:
      fout.write("%d %d PLControllerPVOrpa\n" % (r, r))
  elif opts.ctrl > 0:
    if r in lt_rtrs:
      fout.write("%d %d PLControllerLTOrpa\n" % (r, r))
    else:
      fout.write("%d %d PLControllerPVOrpa\n" % (r, r))

fout.write("subsection policy\n")
for e, rel in rels.iteritems():
  fout.write("%d %d %d %d %d\n" % (e[0], e[0], e[1], e[1], rel))
fout.write("section gsp PLGossiperOrpa\n")
fout.write("section router none\n")
fout.write("section events none\n")
fout.write("# event set-base-time 1229535100\n")
fout.write("# event stop-rcv 1 12\n")
fout.write("# event shut-down 1 15\n")
  
# 400 mx set of LT nodes
# set([0, 1, 3, 4, 5, 6, 7, 8, 9, 12, 13, 15, 16, 18, 19, 22, 23, 24, 25, 27, 30, 31, 34, 35, 36, 37, 38, 39, 42, 43, 48, 49, 55, 61, 62, 66, 67, 69, 71, 72, 74, 75, 78, 79, 81, 82, 84, 85, 89, 93, 94, 95, 97, 100, 102, 104, 106, 107, 109, 110, 112, 115, 116, 117, 118, 119, 120, 124, 128, 130, 131, 132, 134, 135, 136, 138, 140, 143, 149, 150, 151, 152, 154, 155, 156, 159, 161, 163, 167, 170, 176, 177, 180, 181, 182, 184, 187, 188, 190, 191, 193, 194, 196, 201, 202, 208, 209, 210, 212, 213, 216, 217, 218, 219, 223, 225, 233, 234, 241, 245, 250, 251, 253, 254, 256, 257, 258, 260, 261, 263, 265, 267, 268, 269, 272, 274, 276, 278, 281, 288, 290, 292, 293, 294, 296, 297, 299, 300, 302, 304, 306, 307, 308, 309, 311, 312, 313, 318, 319, 320, 321, 324, 326, 327, 328, 331, 334, 344, 346, 347, 351, 352, 353, 356, 358, 359, 360, 362, 363, 368, 369, 370, 371, 372, 375, 378, 379, 380, 384, 385, 386, 387, 390, 391, 392, 393, 394, 395, 396, 398])

