#!/usr/bin/python
import pdb
import time
import random
import pickle
import networkx as NX
import pylab as P
from pl_pl import *

def binom(n):
  b = [0] * (n + 1)
  b[0] = 1
  for i in xrange(1, n + 1):
    b[i] = 1
    j = i - 1
    while j > 0:
      b[j] += b[j - 1]
      j -= 1
  return b

def enum_subsets(s, size):
  # returns a list of subsets
  def helper(theset, subslist, pos, size):
    if len(pos) < size:
      if len(pos) == 0:
        for i in range(0, len(theset)):
          posc = pos[:]
          posc.append(i)
          helper(theset,subslist,posc,size)
      else:
        for i in range(pos[len(pos)-1]+1,len(theset)):
          posc = pos[:]
          posc.append(i)
          helper(theset,subslist,posc,size)
    else:
      subs = set()
      for i in range(0, len(pos)):
        subs.add(theset[pos[i]])
      subslist.append(subs)

  length = len(s)
  subslist = []
  helper(s, subslist, [], size)
  return subslist

def get_max_pairs(params, from_srcs = None, lt_nodes = None):
  g, fg, pls, fts, pllinks, plends, ase2pids, src_set, dst_set, \
      kpids, kgs = params

  # compute reachabilities, evn - egress vnode
  cntd_pairs = 0
  cntd_lt = 0
  cntd_pv = 0
  for src_aid, src_evns in src_set.iteritems():
    if from_srcs is None or src_aid in from_srcs:
      cntd_set = set()
      for evn in src_evns:
        if 1: # XXZ new version
          if fg.has_node(evn): 
            cntd_set.update(dst_set.intersection(NX.dfs_preorder(fg, evn)))
        elif 0: # XXZ original version
          cntd_set.update(dst_set.intersection(NX.dfs_preorder(fg, evn)))
      cntd = len(cntd_set)
      cntd_pairs += cntd
      if lt_nodes is not None:
        if src_aid in lt_nodes:
          cntd_lt += cntd
        else:
          cntd_pv += cntd

  return cntd_pairs, cntd_lt, cntd_pv    

def chooseK(alist, k):
  # returns a list of k distinct elements selected from alist at random. 
  # alist is not changed
  assert k <= len(alist)
  res = []
  for i in range(0,k):
    while True:
      el = random.choice(alist)
      if el not in res:
        res.append(el)
        break
  return res

def figCDF6(d1, d2, d3, f1, f2, f3, title, axis = None, xaxis = None):
  P.figure()
  ld1 = len(d1)
  ld2 = len(d2)
  ld3 = len(d3)
  lf1 = len(f1)
  lf2 = len(f2)
  lf3 = len(f3)
  if xaxis is None:
    P.plot(range(0, len(d1)), d1, label = "LT (all pls)")
    P.plot(range(0, len(d2)), d2, label = "MX (all pls)")
    P.plot(range(0, len(d3)), d3, label = "PV (all pls)")  
    P.plot(range(0, len(f1)), f1, label = "LT (known pls)")
    P.plot(range(0, len(f2)), f2, label = "MX (known pls)")
    P.plot(range(0, len(f3)), f3, label = "PV (known pls)")
  else:
    P.plot(xaxis, d1, label = "LT (all pls)")
    P.plot(xaxis, d2, label = "MX (all pls)")
    P.plot(xaxis, d3, label = "PV (all pls)")  
    P.plot(xaxis, f1, label = "LT (known pls)")
    P.plot(xaxis, f2, label = "MX (known pls)")
    P.plot(xaxis, f3, label = "PV (known pls)")
  if axis is not None:
    P.axis(axis)
  P.legend(loc = "upper left")
  P.title(title)
    

def figCDF(d1, d2, d3, d4, d5, title, dontSort = False, axis = None, xaxis = None):
  P.figure()
  len1 = len(d1)
  len2 = len(d2)
  len3 = len(d3)
  len4 = len(d4)
  len5 = len(d5)
  if not dontSort:
    d1.sort()
    d2.sort()
    d3.sort()
    d4.sort()
    d5.sort()
  if xaxis is None:
    P.plot(range(0, len(d1)), d1, label = "LT")
    P.plot(range(0, len(d2)), d2, label = "MX")
    P.plot(range(0, len(d3)), d3, label = "LT only in MX")  
    P.plot(range(0, len(d4)), d4, label = "PV only in MX")
    P.plot(range(0, len(d5)), d5, label = "PV")
  else:
    P.plot(xaxis, d1, label = "LT")
    P.plot(xaxis, d2, label = "MX")
    P.plot(xaxis, d3, label = "LT only in MX")  
    P.plot(xaxis, d4, label = "PV only in MX")
    P.plot(xaxis, d5, label = "PV")

  if axis is not None:
    P.axis(axis)
  P.legend(loc = "upper left")
  P.title(title)

def figCDFg(d1, d2, d3, d4, d5, d6, title, dontSort = False, 
            axis = None, xaxis = None):
  P.figure()
  len1 = len(d1)
  len2 = len(d2)
  len3 = len(d3)
  len4 = len(d4)
  len5 = len(d5)
  len6 = len(d6)
  if not dontSort:
    d1.sort()
    d2.sort()
    d3.sort()
    d4.sort()
    d5.sort()
    d6.sort()
  if xaxis is None:
    P.plot(range(0, len(d1)), d1, label = "LT")
    P.plot(range(0, len(d2)), d2, label = "MX")
    P.plot(range(0, len(d3)), d3, label = "LT only in MX")  
    P.plot(range(0, len(d4)), d4, label = "PV only in MX")
    P.plot(range(0, len(d5)), d5, label = "PV")
    P.plot(range(0, len(d6)), d6, label = "LT_G")
  else:
    P.plot(xaxis, d1, label = "LT")
    P.plot(xaxis, d2, label = "MX")
    P.plot(xaxis, d3, label = "LT only in MX")  
    P.plot(xaxis, d4, label = "PV only in MX")
    P.plot(xaxis, d5, label = "PV")
    P.plot(xaxis, d6, label = "LT_G")

  if axis is not None:
    P.axis(axis)
  P.legend(loc = "upper left")
  P.title(title)

def getPlEdges(pl):
  # returns the set of inter-AS links that pl goes through.
  es = set()
  as_seq = map(lambda x: x[0], pl.seq)
  last_as = as_seq[0]
  for cas in as_seq:
    if cas != last_as:
      if cas < last_as:
        es.add((cas, last_as))
      else:
        es.add((last_as, cas))
      last_as = cas
  return es

def getHeadLen(pkt):
  plen = 0
  for fid in pkt[3]:
    if fid < 9:
      plen += 0.5
    elif fid < 73:
      plen += 1
    elif fid < 8265:
      plen += 2
    elif fid < 1056841:
      plen += 3
    else:
      plen += 4
  return plen

def getHeadLenOld(pkt):
  plen = 0
  for fid in pkt[3]:
    if fid < 128:
      plen += 1
    elif fid < 16512:
      plen += 2
    elif fid < 2113664:
      plen += 3
    else:
      plen += 4
  return plen

def getMaxHeaderLen(fts, pkt):
  # pkt: [src, dst, nh_lid, fids, ttl]
  # fts: {lid -> {fid -> ([push_fids], nh_lid)}}
  # if error occured, returns -1
  max_len = 0
  dst = pkt[1]
  while True:
    cur_len = getHeadLen(pkt)
    if cur_len > max_len:
      max_len = cur_len

    pkt[4] -= 1
    if pkt[4] < 0:
      print "ttl expired for pkt from", pkt[0], "to", pkt[1]
      #pdb.set_trace()
      return -1

    nh = pkt[2]
    if nh == dst:
      break

    if len(pkt[3]) > 0:
      fid = pkt[3].pop()
    else:
      return -1
    if nh in fts and fid in fts[nh]:
      push_fids, nnh = fts[nh][fid]
      pkt[3].extend(push_fids)
      pkt[2] = nnh
    else:
      return -1
      #print pkt
      #pdb.set_trace()

  return max_len

def getAveHeaderLen(fts, pkt):
  # pkt: [src, dst, nh_lid, fids, ttl]
  # fts: {lid -> {fid -> ([push_fids], nh_lid)}}
  
  hop_count = 0
  tot_len = 0.0
  dst = pkt[1]
  while True:
    hop_count += 1
    tot_len += getHeadLen(pkt)

    pkt[4] -= 1
    if pkt[4] < 0:
      print "ttl expired for pkt from", pkt[0], "to", pkt[1]
      pdb.set_trace()

    nh = pkt[2]
    if nh == dst:
      break

    fid = pkt[3].pop()
    if nh in fts and fid in fts[nh]:
      push_fids, nnh = fts[nh][fid]
      pkt[3].extend(push_fids)
      pkt[2] = nnh
    else:
      print pkt
      pdb.set_trace()

  return tot_len/hop_count
    
def ftSize(params, sep = False):
  # fts: {lid -> {fid -> ([push_fids], nh_lid)}}
  g, fg, pls, fts, pllinks, plends, ase2pids, src_set, dst_set, \
      kpids, kgs = params
  
  tot_ents = {}
  for lid, ft in fts.iteritems():
    aid = lid[0]
    if aid not in tot_ents:
      tot_ents[aid] = 0
    tot_ents[aid] += len(ft)

  if sep:
    tot_ents_lt = []
    tot_ents_pv = []
    for aid, ftlen in tot_ents.iteritems():
      if aid % 2 == 0:
        tot_ents_lt.append(ftlen)
      else:
        tot_ents_pv.append(ftlen)

  if sep:
    return (tot_ents_lt, tot_ents_pv, tot_ents.values())
  else:
    return tot_ents.values()

def headerSize(params, sep = False):
  # we use all the pathlets to find the shortest paths, not only the
  # ones known to the src
  g, fg, pls, fts, pllinks, plends, ase2pids, src_set, dst_set, \
      kpids, kgs = params

  def getFids(path, plends_inv):
    fids = []
    for i in range(0, len(path) - 1):
      fids.append(plends_inv[(path[i], path[i+1])][2])
    fids.reverse()
    return fids

  # build plends inverse
  plends_inv = {}
  for pid, e in plends.iteritems():
    assert e not in plends_inv 
    plends_inv[e] = pid

  # send packets
  all_lens_ave = []
  all_lens_max = []
  if sep:
    all_lens_lt_ave = []
    all_lens_pv_ave = []
    all_lens_lt_max = []
    all_lens_pv_max = []

  for src_aid, src_evns in src_set.iteritems():
    # build shortest paths from each of the egress vnodes
    path_dict_list = []
    for evn in src_evns:
      path_dict_list.append(NX.single_source_dijkstra_path(fg, evn))
    # loop through each destination and send to it if it is reachable
    for dst in dst_set:
      # find the shortest path to dst
      best_path = None
      for epaths in path_dict_list:
        if dst in epaths:
          if (best_path is None) or len(best_path) > len(epaths[dst]):
            best_path = epaths[dst]

      if best_path is not None:
        pkt = [best_path[0], dst, best_path[0], getFids(best_path, plends_inv), 128]
        cur_len_ave = getAveHeaderLen(fts, pkt)
        pkt = [best_path[0], dst, best_path[0], getFids(best_path, plends_inv), 128]
        cur_len_max = getMaxHeaderLen(fts, pkt)

        all_lens_ave.append(cur_len_ave)
        all_lens_max.append(cur_len_max)

        if sep:
          if best_path[0][0] % 2 == 0:
            all_lens_lt_ave.append(cur_len_ave)
            all_lens_lt_max.append(cur_len_max)
          else:
            all_lens_pv_ave.append(cur_len_ave)
            all_lens_pv_max.append(cur_len_max)

  if sep:
    return (all_lens_lt_ave, all_lens_pv_ave, all_lens_ave,
            all_lens_lt_max, all_lens_pv_max, all_lens_max)
  else:
    return (all_lens_ave, all_lens_max)

def precompute(fbase, num_files, top_file, begin_index = 1):

  pls = {} # {pid -> pathlet}
  fts = {} # {lid -> {fid -> ([push_fids], nh_lid)}}
  pllinks = {} # {pid -> set(as_edge_lowest_first)}
  plends = {} # {pid -> (first_lid, last_lid)}
  ase2pids = {} # {as_edge_lowest_first -> set(pids that use this edge)}
                # inverse of pllinks
  known_pids = {} # {aid -> set(pids)}
  known_gs = {} # known graphs  {aid -> digraph using only pls known to aid}

  # get all pls and fts
  for i in range(begin_index, num_files + begin_index):
    f = open("%s_%d.txt" % (fbase, i), "r")
    ps, vlist = pickle.load(f)
    known_pids[i] = set()
    for pid, pl in ps.iteritems():
      known_pids[i].add(pid)
      if pid not in pls:
        pls[pid] = pl
        pllinks[pid] = getPlEdges(pl)
        plends[pid] = (pl.seq[0], pl.seq[-1])
      else:
        if pl.seq != pls[pid].seq:
          print pl.seq, pls[pid].seq
    for lid, ft in vlist:
      fts[lid] = ft
    f.close()

  # build as edge to pls map
  for pid, edges in pllinks.iteritems():
    for e in edges:
      if e not in ase2pids:
        ase2pids[e] = set()
      ase2pids[e].add(pid)

  # build as graph
  g = NX.Graph()
  f = open(top_file, "r")
  for line in f:
    if len(line) == 0 or line[0] == "#":
      continue
    ps = line.split()
    a, b = int(ps[0]), int(ps[1])
    g.add_edge(a,b)
  f.close()

  # build pl graph
  fg = NX.DiGraph()
  for pid, ends in plends.iteritems():
    fg.add_edge(ends[0], ends[1], len(pls[pid].seq) - 1)
  #print "pl graph edge num", len(fg.edges())

  # build known graphs
  if 1:
    for aid, known_pid_set in known_pids.iteritems():
      known_gs[aid] = NX.DiGraph()
      kg = known_gs[aid]
      for pid in known_pid_set: 
        # the weight is removed by the simKnownFailure function
        kg.add_edge(plends[pid][0], plends[pid][1], len(pls[pid].seq) - 1)
      # print "k graph edge num", len(kg.edges())

  # src_set {aid -> [existing egress vnodes]}. Each of the lists is a list of 
  # egress vnodes for an AS that are present and can be used as sources
  src_set = {}
  for aid in g.nodes():
    src_set[aid] = []
    # add all the egress vnodes that aid has
    for vnode_id in [2,4,6]:
      if (aid, vnode_id) in fts:
        src_set[aid].append((aid, vnode_id))

  # set of destination vnodes - (aid, 0)
  dst_set = set(map(lambda x: (x, 0), g.nodes()))

  #if len(plends) != len(set(plends.values())):
  #  pdb.set_trace()
  
  return (g, fg, pls, fts, pllinks, plends, ase2pids, src_set, 
          dst_set, known_pids, known_gs)

def simKnownFailure(params, fedges, from_srcs = None, lt_nodes = None):
  # gets data separate for even (LT nodes) and odd (PV nodes) src
  g, fg, pls, fts, pllinks, plends, ase2pids, src_set, dst_set, \
      kpids, kgs = params

  # get all the pls that depend on fedges
  removed_pids = set()
  for e in fedges:
    if e in ase2pids: # XXZ new version. origianl did not have "if"
      removed_pids.update(ase2pids[e])

  # compute reachabilities
  cntd_pairs = 0
  cntd_lt = 0
  cntd_pv = 0
  for src_aid, src_evns in src_set.iteritems():
    if from_srcs is None or src_aid in from_srcs:
      # remove edges from the graph that src knows
      kg = kgs[src_aid]
      src_kpids = kpids[src_aid]
      edges_to_restore = []
      for pid in removed_pids:
        if pid in src_kpids:
          kg.remove_edge(*plends[pid])
          edges_to_restore.append(plends[pid])
  
      cntd_set = set()
      for evn in src_evns:
        if 1: # XXZ new version
          if kg.has_node(evn): 
            cntd_set.update(dst_set.intersection(NX.dfs_preorder(kg, evn)))
        elif 0: # XXZ original version
          cntd_set.update(dst_set.intersection(NX.dfs_preorder(kg, evn)))
      cntd = len(cntd_set)
   
      cntd_pairs += cntd
      if lt_nodes is not None:
        if src_aid in lt_nodes:
          cntd_lt += cntd
        else:
          cntd_pv += cntd
  
      # restore the graph that src knows
      kg.add_edges_from(edges_to_restore)
  
  return cntd_pairs, cntd_lt, cntd_pv


def simFailure(params, fedges, from_srcs = None, lt_nodes = None):
  # gets data separate for even (LT nodes) and odd (PV nodes) src
  g, fg, pls, fts, pllinks, plends, ase2pids, src_set, dst_set, \
      kpids, kgs = params

  # get all the pls that depend on fedges
  removed_pids = set()
  for e in fedges:
    removed_pids.update(ase2pids[e])

  # remove these pls from the graph
  for pid in removed_pids:
    start, end = plends[pid]
    if not fg.has_edge(start, end):
      print fedge, start, end
      pdb.set_trace()
    fg.remove_edge(start, end)

  # compute reachabilities
  cntd_pairs = 0
  cntd_lt = 0
  cntd_pv = 0
  for src_aid, src_evns in src_set.iteritems():
    if from_srcs is None or src_aid in from_srcs:
      cntd_set = set()
      for evn in src_evns:
        cntd_set.update(dst_set.intersection(NX.dfs_preorder(fg, evn)))
      cntd = len(cntd_set)
   
      cntd_pairs += cntd
      if lt_nodes is not None:
        if src_aid in lt_nodes:
          cntd_lt += cntd
        else:
          cntd_pv += cntd
  
  # restore the graph
  for pid in removed_pids:
    #start, end = plends[pid]
    fg.add_edge(*plends[pid])

  return cntd_pairs, cntd_lt, cntd_pv
  

def read_stats(wdir, run_id, start_rid, stop_rid):
  # dictionaries from router_id to the count
  adv_msg = {}
  wdr_msg = {}
  exp_msg = {}
  pl_count = {}
  pl_mem = {}
  ctrl_mem = {}
  
  # dictionary from router_id to a list of counts
  fwd_ents = {}
  # {rid -> [num_of_samples]}
  hdr_sizes = {}
  
  for rid in range(start_rid, stop_rid+1):
    fname = "%s/stats_%d_%d.txt" % (wdir, run_id, rid)
    try:
      f = open(fname, "r")
    except:
      print "could not open file", fname
      continue
    
    for line in f:
      ps = line.split(":", 1)
      if ps[0] == "ID, sample int":
        assert rid == int(ps[1].split(",")[0])
      elif ps[0] == "Num of rcvd adv msgs":
        adv_msg[rid] = int(ps[1])
      elif ps[0] == "Num of rcvd wdr msgs":
        wdr_msg[rid] = int(ps[1])
      elif ps[0] == "Num of rcvd vnode exp msgs":
        exp_msg[rid] = int(ps[1])
      elif ps[0] == "Max number of known pathlets":
        pl_count[rid] = int(ps[1])
      elif ps[0] == "Max mem to store just pathlets":
        pl_mem[rid] = int(ps[1])
      elif ps[0] == "Max size of ctrl mem in bytes, per pl":
        ctrl_mem[rid] = int(ps[1].split(",")[0])
      elif ps[0] == "Vnode, max fwd entries":
        pps = ps[1].split(",")
        vnode = ( int(pps[0].split("(")[1]), int(pps[1].split(")")[0]) )
        count = int(pps[2])
        if rid not in fwd_ents:
          fwd_ents[rid] = []
        fwd_ents[rid].append(count)
      elif ps[0] == "Dst vnode, hdr sizes":
        pps = ps[1].split(",")
        vnode = ( int(pps[0].split("(")[1]), int(pps[1].split(")")[0]) )
        counts = map(lambda x: int(x), pps[2:])
        if rid not in hdr_sizes:
          hdr_sizes[rid] = []
        for i, count in enumerate(counts):
          if len(hdr_sizes[rid]) < i + 1:
            hdr_sizes[rid].append(0)
          hdr_sizes[rid][i] += count
      elif ps[0] == "Num of local pls":
        # TODO calculate how many pls are created
        pass
      else:
        print "unknown line", line
  
  return (adv_msg, wdr_msg, exp_msg, pl_count, pl_mem, ctrl_mem, 
          fwd_ents, hdr_sizes)
          

def plotMsgCDFs(stats_lt, stats_mx, stats_pv, stats_lt_g = None):
  if stats_lt_g == None:
    # 15 sorted lists, 3 plots of 5 lists each
    adv_lt = stats_lt[0].values()
    wdr_lt = stats_lt[1].values()
    tot_lt = []
    for k in stats_lt[0].iterkeys():
      tot_lt.append(stats_lt[0][k] + stats_lt[1][k])
  
    adv_mx = stats_mx[0].values()
    wdr_mx = stats_mx[1].values()
    tot_mx = []
    for k in stats_mx[0].iterkeys():
      tot_mx.append(stats_mx[0][k] + stats_mx[1][k])
  
    adv_pv = stats_pv[0].values()
    wdr_pv = stats_pv[1].values()
    tot_pv = []
    for k in stats_pv[0].iterkeys():
      tot_pv.append(stats_pv[0][k] + stats_pv[1][k])
  
    adv_mx_lt = []
    adv_mx_pv = []
    wdr_mx_lt = []
    wdr_mx_pv = []
    tot_mx_lt = []
    tot_mx_pv = []
    for k in stats_mx[0].iterkeys():
      if k % 2 == 0:
        # lt node
        adv_mx_lt.append(stats_mx[0][k])
        wdr_mx_lt.append(stats_mx[1][k])
        tot_mx_lt.append(stats_mx[0][k] + stats_mx[1][k])
      else:
        # pv node
        adv_mx_pv.append(stats_mx[0][k])
        wdr_mx_pv.append(stats_mx[1][k])
        tot_mx_pv.append(stats_mx[0][k] + stats_mx[1][k])
  
    figCDF(adv_lt, adv_mx, adv_mx_lt, adv_mx_pv, adv_pv, 
           "CDF of Received Advertisement Msgs")
    figCDF(wdr_lt, wdr_mx, wdr_mx_lt, wdr_mx_pv, wdr_pv, 
           "CDF of Received Withdrawal Msgs")
    figCDF(tot_lt, tot_mx, tot_mx_lt, tot_mx_pv, tot_pv, 
           "CDF of All Received Msgs")
    P.show()

  else:
    # 15 sorted lists, 3 plots of 5 lists each
    adv_lt_g = stats_lt_g[0].values()
    wdr_lt_g = stats_lt_g[1].values()
    tot_lt_g = []
    for k in stats_lt_g[0].iterkeys():
      tot_lt_g.append(stats_lt_g[0][k] + stats_lt_g[1][k])
  
    adv_lt = stats_lt[0].values()
    wdr_lt = stats_lt[1].values()
    tot_lt = []
    for k in stats_lt[0].iterkeys():
      tot_lt.append(stats_lt[0][k] + stats_lt[1][k])
  
    adv_mx = stats_mx[0].values()
    wdr_mx = stats_mx[1].values()
    tot_mx = []
    for k in stats_mx[0].iterkeys():
      tot_mx.append(stats_mx[0][k] + stats_mx[1][k])
  
    adv_pv = stats_pv[0].values()
    wdr_pv = stats_pv[1].values()
    tot_pv = []
    for k in stats_pv[0].iterkeys():
      tot_pv.append(stats_pv[0][k] + stats_pv[1][k])
  
    adv_mx_lt = []
    adv_mx_pv = []
    wdr_mx_lt = []
    wdr_mx_pv = []
    tot_mx_lt = []
    tot_mx_pv = []
    for k in stats_mx[0].iterkeys():
      if k % 2 == 0:
        # lt node
        adv_mx_lt.append(stats_mx[0][k])
        wdr_mx_lt.append(stats_mx[1][k])
        tot_mx_lt.append(stats_mx[0][k] + stats_mx[1][k])
      else:
        # pv node
        adv_mx_pv.append(stats_mx[0][k])
        wdr_mx_pv.append(stats_mx[1][k])
        tot_mx_pv.append(stats_mx[0][k] + stats_mx[1][k])
  
    figCDFg(adv_lt, adv_mx, adv_mx_lt, adv_mx_pv, adv_pv, adv_lt_g,
           "CDF of Received Advertisement Msgs")
    figCDFg(wdr_lt, wdr_mx, wdr_mx_lt, wdr_mx_pv, wdr_pv, wdr_lt_g,
           "CDF of Received Withdrawal Msgs")
    figCDFg(tot_lt, tot_mx, tot_mx_lt, tot_mx_pv, tot_pv, tot_lt_g,
           "CDF of All Received Msgs")
    P.show()


def plotCtrlMem(params_lt, params_mx, params_pv, params_lt_g = None):
  if params_lt_g == None:
    pl_mem_lt = params_lt[4].values()
    pl_mem_mx = params_mx[4].values()
    pl_mem_pv = params_pv[4].values()
    ctrl_mem_lt = params_lt[5].values()
    ctrl_mem_mx = params_mx[5].values()
    ctrl_mem_pv = params_pv[5].values()
    pl_mem_lt.sort()
    pl_mem_mx.sort()
    pl_mem_pv.sort()
    ctrl_mem_lt.sort()
    ctrl_mem_mx.sort()
    ctrl_mem_pv.sort()
  
    P.figure()
    P.plot(range(0, len(pl_mem_lt)), pl_mem_lt, label = "LT")
    P.plot(range(0, len(pl_mem_mx)), pl_mem_mx, label = "MX")
    P.plot(range(0, len(pl_mem_pv)), pl_mem_pv, label = "PV")
    P.title("Mem to store just pls")
    P.legend(loc = "upper left")
  
    P.figure()
    P.plot(range(0, len(ctrl_mem_lt)), ctrl_mem_lt, label = "LT")
    P.plot(range(0, len(ctrl_mem_mx)), ctrl_mem_mx, label = "MX")
    P.plot(range(0, len(ctrl_mem_pv)), ctrl_mem_pv, label = "PV")
    P.title("Total Ctrl Mem")
    P.legend(loc = "upper left")
  
  else:
    pl_mem_lt = params_lt[4].values()
    pl_mem_lt_g = params_lt_g[4].values()
    pl_mem_mx = params_mx[4].values()
    pl_mem_pv = params_pv[4].values()
    ctrl_mem_lt = params_lt[5].values()
    ctrl_mem_lt_g = params_lt_g[5].values()
    ctrl_mem_mx = params_mx[5].values()
    ctrl_mem_pv = params_pv[5].values()
    pl_mem_lt.sort()
    pl_mem_lt_g.sort()
    pl_mem_mx.sort()
    pl_mem_pv.sort()
    ctrl_mem_lt.sort()
    ctrl_mem_lt_g.sort()
    ctrl_mem_mx.sort()
    ctrl_mem_pv.sort()
  
    P.figure()
    P.plot(range(0, len(pl_mem_lt)), pl_mem_lt, label = "LT")
    P.plot(range(0, len(pl_mem_lt_g)), pl_mem_lt_g, label = "LT_G")
    P.plot(range(0, len(pl_mem_mx)), pl_mem_mx, label = "MX")
    P.plot(range(0, len(pl_mem_pv)), pl_mem_pv, label = "PV")
    P.title("Mem to store just pls")
    P.legend(loc = "upper left")
  
    P.figure()
    P.plot(range(0, len(ctrl_mem_lt)), ctrl_mem_lt, label = "LT")
    P.plot(range(0, len(ctrl_mem_lt_g)), ctrl_mem_lt_g, label = "LT_G")
    P.plot(range(0, len(ctrl_mem_mx)), ctrl_mem_mx, label = "MX")
    P.plot(range(0, len(ctrl_mem_pv)), ctrl_mem_pv, label = "PV")
    P.title("Total Ctrl Mem")
    P.legend(loc = "upper left")
  
def plot1(params, a,b,c,d,e,f,g,h):
  adv_msg, wdr_msg, exp_msg, pl_count, pl_mem, ctrl_mem, \
      fwd_ents, hdr_sizes = params
  if a:
    am = adv_msg.values()
    am.sort()
    print "received adv", sum(am)/len(am)
    P.plot(range(0, len(am)), am)
    P.title("received adv")
  if b:  
    P.figure()
    wm = wdr_msg.values()
    wm.sort()
    print "received wdr", sum(wm)/len(wm)
    P.plot(range(0, len(wm)), wm)
    P.title("received wdr")
  if c:
    P.figure()
    em = exp_msg.values()
    em.sort()
    print "exposed vnodes", sum(em)/len(em)
    P.plot(range(0, len(em)), em)
    P.title("exposed vnodes")
  if d:
    P.figure()
    pc = pl_count.values()
    pc.sort()
    print "pl_count", sum(pc)/len(pc)
    P.plot(range(0, len(pc)), pc)
    P.title("pl count")
  if e:
    P.figure()
    pm = pl_mem.values()
    pm.sort()
    print "pl_mem", sum(pm)/len(pm)
    P.plot(range(0, len(pm)), pm)
    P.title("mem just for pls")
  if f:
    P.figure()
    cm = ctrl_mem.values()
    cm.sort()
    print "ctrl_mem", sum(cm)/len(cm)
    P.plot(range(0, len(cm)), cm)
    P.title("total ctrl mem")
  if g:
    P.figure()
    fents = []
    for lc in fwd_ents.itervalues():
      fents.extend(lc)
    fents.sort()
    print "average fwd entries per vnode", float(sum(fents))/len(fents)
    P.plot(range(0, len(fents)), fents)
    P.title("fwd entries per vnode")
  if h:
    P.figure()
    ## {rid -> [num_of_samples]}
    #hdr_sizes = {}
    hdrs = []
    totlen = 0
    for ns in hdr_sizes.itervalues():
      for i,v in enumerate(ns):
        if len(hdrs) < i + 1:
          hdrs.append(0)
        hdrs[i] += ns[i]
        totlen += i*ns[i]
    print hdrs
    print "average hdr len", totlen/float(sum(hdrs))
    P.plot(range(0, len(hdrs)), hdrs)
    P.title("hdr len")
    
  P.show()

def smoothly_expand(alist, num):
  # alist is a sorted list of number of length less than num. Returns a new
  # list of length num that is similar to alist
  flist = []
  clen = len(alist)
  assert clen < num
  q = num / clen
  r = num % clen
  r_count = 0
  for elem in alist:
    flist.extend([elem]*q)
    if r_count < r:
      flist.append(elem)
      r_count += 1
  assert len(flist) == num
  return flist

def get_lt_nodes(fname):
  f = open(fname, "r")
  s = []
  for line in f:
    ps = line.split()
    if (len(ps) == 3 and ps[2] == "PLControllerLTOrpa" and
        ps[0] != "section"):
      s.append(int(ps[0]))
  f.close()
  return set(s)

def get_pv_nodes(fname):
  f = open(fname, "r")
  s = []
  for line in f:
    ps = line.split()
    if (len(ps) == 3 and ps[2] == "PLControllerPVOrpa" and
        ps[0] != "section"):
      s.append(int(ps[0]))
  f.close()
  return set(s)


if __name__ == "__main__":

  if 0:
    # CONNECTIVITY DATA FOR 79 PV and 1 LT
    random.seed(1379173*223 % 173571)
    iters = 4000
    num_failures = 5
    ratios = []
    for run in range(6, 41):
      #for run in range(7, 9):
      lt_num = 1
      stime = time.time()
      params = precompute("/home/igor/pl/run/80/80_%d_%d/80_dump" % (run, lt_num),
                          80, "/home/igor/pl/conf/80.types", 0)
      lt_nodes = get_lt_nodes("/home/igor/pl/conf/80/80n_%d_%d.txt" % (run, lt_num))
      assert len(lt_nodes) == lt_num
      edge_list = params[0].edges()
      max_pairs, tmp, tmp = get_max_pairs(params, lt_nodes)
      tot_cntd = 0
      for j in range(0, iters):
        random_fail_set = chooseK(edge_list, num_failures)
        #x, y, z = simKnownFailure(params, random_fail_set, lt_nodes)
        x, y, z = simFailure(params, random_fail_set, lt_nodes)
        tot_cntd += x
  
      discn_ratio = 1 - float(tot_cntd)/(max_pairs*iters)
      ratios.append(discn_ratio)
      print discn_ratio, max_pairs, time.time() - stime
    print ratios, sum(ratios)/len(ratios)

  if 1:
    # CONNECTIVITY DATA FOR 80 PV
    #random.seed(1379173*123 % 173571)
    lt_nums = [1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,18,21,24,27,30,33,36,39]
    all_ratios = []
    iters = 200
    num_failures = 5
    for run in range(1, 4):
      ratios = []
      for lt_num in lt_nums:
        stime = time.time()
        #print "precomputing", lt_num, run
        params = precompute("/home/igor/pl/run/80/80_%d_%d/80_dump" % (run, lt_num),
                            80, "/home/igor/pl/conf/80.types", 0)
        pv_nodes = get_pv_nodes("/home/igor/pl/conf/80/80n_%d_%d.txt" % (run, lt_num))
        assert 80 - len(pv_nodes) == lt_num
        edge_list = params[0].edges()
        max_pairs, tmp, tmp = get_max_pairs(params, pv_nodes)
        tot_cntd = 0
        for j in range(0, iters):
          random_fail_set = chooseK(edge_list, num_failures)
          #x, y, z = simKnownFailure(params, random_fail_set, pv_nodes)
          x, y, z = simFailure(params, random_fail_set, pv_nodes)
          tot_cntd += x
    
        discn_ratio = 1 - float(tot_cntd)/(max_pairs*iters)
        ratios.append(discn_ratio)
        print discn_ratio, max_pairs, time.time() - stime
    
      all_ratios.append(ratios)
      print ratios
      P.figure()
      P.plot(lt_nums, ratios)
      P.title("Run %d, Num Failures %d, X-axis number of LT nodes, "\
                "Y-axis ratio of nodes PV nodes cannot reach." % (run, num_failures))
      P.savefig("/home/igor/pl/data/lt_ratios/80_frompv_all_%d_%d.png" % (run, num_failures))
    
    print all_ratios
    # get the average of all ratios
    ave_ratios = []
    for ltn in range(0, len(lt_nums)):
      ave_ratios.append(0)
      for run in range(0, 5):
        ave_ratios[-1] += all_ratios[run][ltn]
      ave_ratios[-1] = ave_ratios[-1] / 5.0
  
    P.figure()
    P.plot(lt_nums, ave_ratios)
    P.title("Average of 5 runs")
    P.savefig("/home/igor/pl/data/lt_ratios/80_frompv_all_ave_%d.png" % (num_failures))
  
  
  if 0:
    # CONNECTIVITY DATA FOR 80 LT
    lt_nodes = {} # {lt_num -> set(lt nodes)}
    lt_nodes[1] = set([4])
  
    lt_nums = [1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,18,21,24,27,30,33,36,39]
    all_ratios = []
    iters = 200
    num_failures = 5
    for run in range(4, 6):
      ratios = []
      for lt_num in lt_nums:
        stime = time.time()
        params = precompute("/home/igor/pl/run/80/80_%d_%d/80_dump" % (run, lt_num),
                            80, "/home/igor/pl/conf/80.types", 0)
        lt_nodes = get_lt_nodes("/home/igor/pl/conf/80/80n_%d_%d.txt" % (run, lt_num))
        assert len(lt_nodes) == lt_num
        edge_list = params[0].edges()
        max_pairs, tmp, tmp = get_max_pairs(params, lt_nodes)
        tot_cntd = 0
        #random.seed(1379173*run % 173571)
        for j in range(0, iters):
          random_fail_set = chooseK(edge_list, num_failures)
          #x, y, z = simKnownFailure(params, random_fail_set, lt_nodes)
          x, y, z = simFailure(params, random_fail_set, lt_nodes)
          tot_cntd += x
    
        discn_ratio = 1 - float(tot_cntd)/(max_pairs*iters)
        ratios.append(discn_ratio)
        print discn_ratio, max_pairs, time.time() - stime
      
      print ratios
      all_ratios.append(ratios)
      P.figure()
      P.plot(lt_nums, ratios)
      P.title("Run %d, Num Failures %d, X-axis number of LT nodes, "\
                "Y-axis ratio of nodes LT nodes cannot reach." % (run, num_failures))
      P.savefig("/home/igor/pl/data/lt_ratios/80_all_%d_%d.png" % (run, num_failures))
    
  
    # get the average of all ratios
    ave_ratios = []
    for ltn in range(0, len(lt_nums)):
      ave_ratios.append(0)
      for run in range(0, 5):
        ave_ratios[-1] += all_ratios[run][ltn]
      ave_ratios[-1] = ave_ratios[-1] / 5.0
  
    print all_ratios
    P.figure()
    P.plot(lt_nums, ave_ratios)
    P.title("Average of 5 runs")
    P.savefig("/home/igor/pl/data/lt_ratios/80_all_ave_%d.png" % (num_failures))
    
  
  if 0:
    # ALL GRAPHS DEPENDENT ON STATS
  
    # get stats
    if 1:
      stats_lt = read_stats("/home/igor/pl/run/log/400_all_f/lt_g", 123, 0, 399)
      stats_pv = read_stats("/home/igor/pl/run/log/400_all_f/pv", 123, 0, 399)
    elif 0:
      stats_lt = read_stats("/home/igor/pl/run/log/400_lt", 123, 0, 399)
      stats_mx = read_stats("/home/igor/pl/run/log/400_mx", 123, 0, 399)
      stats_pv = read_stats("/home/igor/pl/run/log/400_pv", 123, 0, 399)
    elif 0:
      stats_lt_g = read_stats("/home/igor/pl/run/log/63_lt_g", 123, 1, 63)
      stats_lt = read_stats("/home/igor/pl/run/log/63_lt", 123, 1, 63)
      stats_mx = read_stats("/home/igor/pl/run/log/63_mx", 123, 1, 63)
      stats_pv = read_stats("/home/igor/pl/run/log/63_pv", 123, 1, 63)
    print "got stats"
  
    if 0:
      # plot 2 ctrl memory plots
      plotCtrlMem(stats_lt, stats_mx, stats_pv, stats_lt_g)
      print "ploted ctrl mem"
      P.show()
      raw_input("press enter to continue")
  
    if 0:
      plotMsgCDFs(stats_lt, stats_mx, stats_pv, stats_lt_g)
      P.show()
      raw_input("press enter to continue")
           
    
  
  if 0:
    # ALL GRAPHS DEPENDENT ON DUMP
  
    # get the params
    if 0:
      params_lt = precompute("/home/igor/pl/run/400_lt/400_lt_dump", 400, 
                             "/home/igor/pl/conf/400.types", 0)
      print "lt done"
      params_mx = precompute("/home/igor/pl/run/400_mx/400_mx_dump", 400, 
                             "/home/igor/pl/conf/400.types", 0)
      print "mx done"
      params_pv = precompute("/home/igor/pl/run/400_pv/400_pv_dump", 400, 
                             "/home/igor/pl/conf/400.types", 0)
      print "pv done"
    else:
      params_lt = precompute("/home/igor/pl/run/63_lt/63_lt_dump", 63, 
                             "/home/igor/pl/conf/63.types")
      params_mx = precompute("/home/igor/pl/run/63_mx/63_mx_dump", 63, 
                             "/home/igor/pl/conf/63.types")
      params_pv = precompute("/home/igor/pl/run/63_pv/63_pv_dump", 63, 
                             "/home/igor/pl/conf/63.types")
      params_lt_g = precompute("/home/igor/pl/run/63_lt_g/63_lt_g_dump", 63, 
                               "/home/igor/pl/conf/63.types")
   
    if 0:
      # plot ft table sizes
      tot_ents_lt = ftSize(params_lt) 
      tot_ents_mx_lt, tot_ents_mx_pv, tot_ents_mx = ftSize(params_mx, sep = True)
      tot_ents_pv = ftSize(params_pv)
      tot_ents_lt_g = ftSize(params_lt_g)
  
      figCDFg(tot_ents_lt, tot_ents_mx, tot_ents_mx_lt, 
             tot_ents_mx_pv, tot_ents_pv, tot_ents_lt_g,
             "Router Fowarding Table Size CDFs")
      print "plotted ft size"
      P.show()
      raw_input("Press enter to continue")
  
    if 0:
      # plot header sizes
      lens_lt_ave, lens_lt_max = headerSize(params_lt) 
      lens_mx_lt_ave, lens_mx_pv_ave, lens_mx_ave, \
          lens_mx_lt_max, lens_mx_pv_max, lens_mx_max = \
          headerSize(params_mx, sep = True)
      lens_pv_ave, lens_pv_max = headerSize(params_pv)
      lens_lt_g_ave, lens_lt_g_max = headerSize(params_lt_g)
      
      figCDFg(lens_lt_ave, lens_mx_ave, lens_mx_lt_ave, lens_mx_pv_ave, lens_pv_ave,
             lens_lt_g_ave, "Average Header Length CDFs")
      figCDFg(lens_lt_max, lens_mx_max, lens_mx_lt_max, lens_mx_pv_max, lens_pv_max,
             lens_lt_g_max, "Maximum Header Length CDFs")
      print "plotted header size"
      P.show()
      raw_input("Press enter to continue")
  
    if 1:
      # get disconnectivity ratios
      random.seed(123)
      iters = 10
      if 0:
        # precomputed results for 400
        max_pairs_lt = (155016, 76822, 78194) # get_max_pairs(params_lt)
        max_pairs_mx = (155016, 76822, 78194) # get_max_pairs(params_mx)
        max_pairs_pv = (155016, 76822, 78194) # get_max_pairs(params_pv)
      else:
        max_pairs_lt = get_max_pairs(params_lt)
        max_pairs_mx = get_max_pairs(params_mx)
        max_pairs_pv = get_max_pairs(params_pv)
        max_pairs_lt_g = get_max_pairs(params_lt_g)
  
      print "getting discon: max_pairs:", max_pairs_lt, \
          max_pairs_mx, max_pairs_pv, max_pairs_lt_g
    
      edge_list = params_lt[0].edges()
      res_tps = []
      for num_fs in range(1, 40, 3):
        tot_cntd_lt = 0
        tot_cntd_lt_g = 0
        tot_cntd_mx = 0
        tot_cntd_mx_pv = 0
        tot_cntd_mx_lt = 0
        tot_cntd_pv = 0
        for j in range(0, iters):
          #if j % 3 == 0:
          #  print num_fs, j
          stime = time.time()
          random_fail_set = chooseK(edge_list, num_fs)
          x, y, z = simKnownFailure(params_lt, random_fail_set)
          #x, y, z = simFailure(params_lt, random_fail_set)
          tot_cntd_lt += x
          x, y, z = simKnownFailure(params_lt_g, random_fail_set)
          #x, y, z = simFailure(params_lt_g, random_fail_set)
          tot_cntd_lt_g += x
          x, y, z= simKnownFailure(params_pv, random_fail_set)
          #x, y, z= simFailure(params_pv, random_fail_set)
          tot_cntd_pv += x
          x, y, z = simKnownFailure(params_mx, random_fail_set)
          #x, y, z = simFailure(params_mx, random_fail_set)
          tot_cntd_mx += x
          tot_cntd_mx_lt += y
          tot_cntd_mx_pv += z
          if j == 0:
            print time.time() - stime, "seconds per iteration"
  
        res_tp = (num_fs, 
                  float(tot_cntd_lt)/(max_pairs_lt[0]*iters),
                  float(tot_cntd_mx)/(max_pairs_mx[0]*iters),
                  float(tot_cntd_mx_lt)/(max_pairs_mx[1]*iters),
                  float(tot_cntd_mx_pv)/(max_pairs_mx[2]*iters),
                  float(tot_cntd_pv)/(max_pairs_pv[0]*iters),
                  float(tot_cntd_lt_g)/(max_pairs_lt_g[0]*iters))
        res_tps.append(res_tp)
        print res_tp
      print res_tps, time.time() - stime
  
      c_lt = []
      c_lt_g = []
      c_mx = []
      c_mx_lt = []
      c_mx_pv = []
      c_pv = []
      x_axis = []
      for i in range(0, len(res_tps)):
        x_axis.append(res_tps[i][0])
        c_lt.append(1 - res_tps[i][1])
        c_mx.append(1 - res_tps[i][2])
        c_mx_lt.append(1 - res_tps[i][3])
        c_mx_pv.append(1 - res_tps[i][4])
        c_pv.append(1 - res_tps[i][5])
        c_lt_g.append(1 - res_tps[i][6])
  
      figCDFg(c_lt, c_mx, c_mx_lt, c_mx_pv, c_pv, c_lt_g,
             "Portion of disconnected pair versus number of failed links", 
             dontSort = True, xaxis = x_axis)
  
      P.show()
      raw_input("Input something to exit:")
  
  if 0:
    params_lt = read_stats("/home/igor/pl/run/log/63_lt", 123, 1, 63)
    params_mx = read_stats("/home/igor/pl/run/log/63_mx", 123, 1, 63)
    params_pv = read_stats("/home/igor/pl/run/log/63_pv", 123, 1, 63)
    plotCtrlMem(params_lt, params_mx, params_pv)
    P.show()
  
  if 0:
    params_lt = precompute("/home/igor/pl/run/63_lt/63_lt_dump", 63, 
                           "/home/igor/pl/conf/63.types")
    params_mx = precompute("/home/igor/pl/run/63_mx/63_mx_dump", 63, 
                           "/home/igor/pl/conf/63.types")
    params_pv = precompute("/home/igor/pl/run/63_pv/63_pv_dump", 63, 
                           "/home/igor/pl/conf/63.types")
    tot_ents_lt = ftSize(params_lt) 
    tot_ents_mx_lt, tot_ents_mx_pv, tot_ents_mx = ftSize(params_mx, sep = True)
    tot_ents_pv = ftSize(params_pv)
  
    figCDF(tot_ents_lt, tot_ents_mx, tot_ents_mx_lt, tot_ents_mx_pv, tot_ents_pv,
           "Router Fowarding Table Size CDFs")
    P.show()
  
  if 0:
    params_lt = precompute("/home/igor/pl/run/63_lt/63_lt_dump", 63, 
                           "/home/igor/pl/conf/63.types")
    params_mx = precompute("/home/igor/pl/run/63_mx/63_mx_dump", 63, 
                           "/home/igor/pl/conf/63.types")
    params_pv = precompute("/home/igor/pl/run/63_pv/63_pv_dump", 63, 
                           "/home/igor/pl/conf/63.types")
    print "lt, mx, pv"
    lens_lt_ave, lens_lt_max = headerSize(params_lt) 
    lens_mx_lt_ave, lens_mx_pv_ave, lens_mx_ave, \
        lens_mx_lt_max, lens_mx_pv_max, lens_mx_max = \
        headerSize(params_mx, sep = True)
    lens_pv_ave, lens_pv_max = headerSize(params_pv)
  
    figCDF(lens_lt_ave, lens_mx_ave, lens_mx_lt_ave, lens_mx_pv_ave, lens_pv_ave,
           "Average Header Length CDFs")
    figCDF(lens_lt_max, lens_mx_max, lens_mx_lt_max, lens_mx_pv_max, lens_pv_max,
           "Maximum Header Length CDFs")
  
    P.show()
  
  
  if 0:
    random.seed(123)
    iters = 20
    if 1:    
      params_lt = precompute("/home/igor/pl/run/400_lt/400_lt_dump", 400, 
                             "/home/igor/pl/conf/400.types", 0)
      params_mx = precompute("/home/igor/pl/run/400_mx/400_mx_dump", 400, 
                             "/home/igor/pl/conf/400.types", 0)
      params_pv = precompute("/home/igor/pl/run/400_pv/400_pv_dump", 400, 
                             "/home/igor/pl/conf/400.types", 0)
    else:
      params_lt = precompute("/home/igor/pl/run/63_lt/63_lt_dump", 63, 
                             "/home/igor/pl/conf/63.types")
      params_mx = precompute("/home/igor/pl/run/63_mx/63_mx_dump", 63, 
                             "/home/igor/pl/conf/63.types")
      params_pv = precompute("/home/igor/pl/run/63_pv/63_pv_dump", 63, 
                             "/home/igor/pl/conf/63.types")
    max_pairs_lt = get_max_pairs(params_lt)
    max_pairs_mx = get_max_pairs(params_mx)
    max_pairs_pv = get_max_pairs(params_pv)
    print " max_pairs:", max_pairs_lt, max_pairs_mx, max_pairs_pv
    
    edge_list = params_lt[0].edges()
    res_tps = []
    print "num_fs, lt, mx, mx_lt, mx_pv, pv:"
    stime = time.time()
    for num_fs in range(140, 141, 10):
      tot_cntd_lt = 0
      tot_cntd_mx = 0
      tot_cntd_mx_pv = 0
      tot_cntd_mx_lt = 0
      tot_cntd_pv = 0
      for j in range(0, iters):
        if j % 50 == 0:
          print num_fs, j
        random_fail_set = chooseK(edge_list, num_fs)
        #x, y, z = simKnownFailure(params_lt, random_fail_set)
        x, y, z = simFailure(params_lt, random_fail_set)
        tot_cntd_lt += x
        #x, y, z= simKnownFailure(params_pv, random_fail_set)
        x, y, z= simFailure(params_pv, random_fail_set)
        tot_cntd_pv += x
        #x, y, z = simKnownFailure(params_mx, random_fail_set)
        x, y, z = simFailure(params_mx, random_fail_set)
        tot_cntd_mx += x
        tot_cntd_mx_lt += y
        tot_cntd_mx_pv += z
  
      res_tp = (num_fs, 
                float(tot_cntd_lt)/(max_pairs_lt[0]*iters),
                float(tot_cntd_mx)/(max_pairs_mx[0]*iters),
                float(tot_cntd_mx_lt)/(max_pairs_mx[1]*iters),
                float(tot_cntd_mx_pv)/(max_pairs_mx[2]*iters),
                float(tot_cntd_pv)/(max_pairs_pv[0]*iters))
      res_tps.append(res_tp)
      print res_tp
    print res_tps, time.time() - stime
  
  
  if 0:
    data63 = """1 0.99974804 0.99160997 0.9983512 0.9850793 0.984555303603
    2 0.998347190728 0.984265558075 0.995023041475 0.973844246032 0.970864197531
    3 0.997954144621 0.973801965231 0.99273937532 0.955456349206 0.95579239103
    4 0.996381960191 0.965311161502 0.99082437276 0.940595238095 0.939843789368
    5 0.994678760393 0.955928445452 0.98523297491 0.92753968254 0.927694633409
    6 0.994441924918 0.946712018141 0.983538146441 0.911036706349 0.911383219955
    7 0.992068531116 0.936895943563 0.978847926267 0.896254960317 0.898281683044
    8 0.989438145629 0.92865457294 0.973727598566 0.884990079365 0.886034265558
    9 0.987518266566 0.919110607206 0.969180747568 0.87060515873 0.873025951121
    10 0.988042328042 0.91248425296 0.965965181772 0.860674603175 0.859428067523
    11 0.986581002771 0.904333585286 0.96394265233 0.846587301587 0.845978835979
    12 0.981224489796 0.894116906022 0.954480286738 0.835639880952 0.833834719073
    13 0.978080120937 0.883295540438 0.949068100358 0.819578373016 0.817676996725
    14 0.97539430587 0.869329805996 0.941382488479 0.799528769841 0.799632149156
    15 0.976830435878 0.869503653313 0.942457757296 0.798829365079 0.794411690602
    16 0.970173847317 0.857717309146 0.932365591398 0.785401785714 0.782899974805
    17 0.96455026455 0.843293020912 0.924992319508 0.764146825397 0.763731418493
    18 0.965754598136 0.837654320988 0.924091141833 0.753918650794 0.75609473419
    19 0.957369614512 0.82527084908 0.910076804916 0.743115079365 0.744162257496
    20 0.952406147644 0.814363819602 0.903579109063 0.727936507937 0.726749811036
    21 0.946928697405 0.804320987654 0.893492063492 0.717936507937 0.71656840514
    22 0.946097253716 0.802401108592 0.890937019969 0.716631944444 0.71146888385
    23 0.939254220207 0.785165028975 0.876231438812 0.696944444444 0.694195011338
    24 0.940231796422 0.779468379945 0.874336917563 0.687564484127 0.684257999496
    25 0.930587049635 0.766553287982 0.860112647209 0.67591765873 0.668742756362
    26 0.931388259007 0.758775510204 0.859795186892 0.660912698413 0.659720332577
    27 0.923063744016 0.744696397077 0.848750640041 0.643893849206 0.646386999244
    28 0.916472663139 0.739758125472 0.838571428571 0.644032738095 0.637724867725
    29 0.909634668682 0.732164273117 0.831689708141 0.635749007937 0.627540942303
    30 0.900912068531 0.715404383976 0.817403993856 0.616592261905 0.614318468128
    31 0.891987906274 0.702103804485 0.806513056836 0.60095734127 0.59837994457
    32 0.896895943563 0.699271856891 0.807296466974 0.594623015873 0.595036533132
    33 0.883950617284 0.68174351222 0.787962109575 0.578844246032 0.578329554044
    34 0.88559334845 0.682872260015 0.791362007168 0.57777281746 0.575691609977
    35 0.874134542706 0.667140337617 0.779989759345 0.557817460317 0.558296800202
    36 0.8620861678 0.657548500882 0.767834101382 0.550709325397 0.552534643487
    37 0.853686067019 0.649410430839 0.751464413722 0.550545634921 0.542229780801
    38 0.848480725624 0.637026958932 0.73975422427 0.537509920635 0.532317964223
    39 0.840448475687 0.628092718569 0.730614439324 0.528774801587 0.520544217687
    40 0.834658604182 0.614643487024 0.725289298515 0.507455357143 0.509274376417
    41 0.825679012346 0.608878810784 0.708986175115 0.511899801587 0.501106072058
    42 0.812426303855 0.599841269841 0.706543778802 0.496473214286 0.491945074326
    43 0.807825648778 0.585180146133 0.690117767537 0.483521825397 0.480574452003
    44 0.805749559083 0.581428571429 0.685821812596 0.480297619048 0.474693877551
    45 0.79251196775 0.564033761653 0.666528417819 0.464742063492 0.457994457042
    46 0.777631645251 0.553771730915 0.659380440348 0.451463293651 0.448049886621
    47 0.773287981859 0.551143864953 0.650563236047 0.454831349206 0.449826152683
    48 0.767004283195 0.541997984379 0.641515616999 0.445590277778 0.438939279415
    49 0.755207860922 0.532013101537 0.627511520737 0.439499007937 0.42855883094
    50 0.753096497858 0.519239103049 0.618832565284 0.422757936508 0.41474930713
    51 0.731652809272 0.506487780297 0.606013824885 0.410071924603 0.407646762409
    52 0.724633408919 0.49974930713 0.598484383001 0.404099702381 0.4019904258
    53 0.718294280675 0.491528092719 0.585752688172 0.400248015873 0.394001007811
    54 0.704759385236 0.481869488536 0.57547875064 0.391185515873 0.385587049635
    55 0.699236583522 0.470512723608 0.563671274962 0.380265376984 0.374569160998
    56 0.684968505921 0.462325522802 0.55582437276 0.371748511905 0.367847064752
    57 0.681161501638 0.453849836231 0.541758832565 0.368687996032 0.358648274124
    58 0.666900982615 0.442700932225 0.533791602663 0.354456845238 0.349952129
    59 0.658987150416 0.435532879819 0.524372759857 0.349469246032 0.343742756362
    60 0.650037792895 0.427125220459 0.51605734767 0.340972222222 0.337653061224
    61 0.64246409675 0.418306878307 0.501546338966 0.337668650794 0.330076845553
    62 0.629765684051 0.408660871756 0.490949820789 0.328943452381 0.32126606198
    63 0.609624590577 0.39589569161 0.474813108039 0.319444444444 0.311834215168
    64 0.609609473419 0.391279919375 0.472035330261 0.313048115079 0.30783446712
    65 0.58673973293 0.379871504157 0.458753200205 0.303454861111 0.299382716049
    66 0.584018644495 0.375491307634 0.450220174091 0.303097718254 0.295878054926
    67 0.572297808012 0.364899218947 0.440189452125 0.291961805556 0.285898211136
    68 0.552731166541 0.353373645755 0.426172555044 0.282849702381 0.277223481985
    69 0.541340388007 0.345701688083 0.417524321557 0.276123511905 0.272024439405
    70 0.531617535903 0.340589569161 0.405097286226 0.278097718254 0.269174855127
    71 0.52733686067 0.333140589569 0.400596518177 0.26779265873 0.261313932981
    72 0.510388007055 0.321550768456 0.386216077829 0.25890625 0.252142857143
    73 0.498092718569 0.315464852608 0.379001536098 0.253913690476 0.247736205593
    74 0.485742000504 0.307271352986 0.367736815156 0.248695436508 0.241565885614
    75 0.466855631141 0.294685059209 0.351897081413 0.239260912698 0.232373393802
    76 0.472169312169 0.292928949358 0.355558115719 0.232256944444 0.227631645251
    77 0.452839506173 0.280098261527 0.336105990783 0.22584077381 0.221077097506
    78 0.443076341648 0.274051398337 0.329679979519 0.220161210317 0.215537918871
    79 0.432433862434 0.266907281431 0.318376856119 0.217046130952 0.210347694633
    80 0.415479969766 0.258522297808 0.307747055812 0.210835813492 0.203689846309
    81 0.402144116906 0.252238599143 0.298594470046 0.207331349206 0.19947845805
    82 0.390657596372 0.245158730159 0.290112647209 0.201609623016 0.194575459814
    83 0.383000755858 0.238791887125 0.28331797235 0.195657242063 0.189382716049
    84 0.375742000504 0.232792894936 0.274976958525 0.191927083333 0.184671201814
    85 0.361597379693 0.225490047871 0.267969790067 0.184337797619 0.178499622071
    86 0.353887629126 0.220415721844 0.259418842806 0.182631448413 0.175272108844
    87 0.334016124969 0.210570672714 0.24886328725 0.173474702381 0.167223481985
    88 0.322842025699 0.202887377173 0.238028673835 0.168844246032 0.161657848325
    89 0.318059964727 0.198054925674 0.233748079877 0.16347718254 0.158149407911
    90 0.299967246158 0.190617283951 0.225227854583 0.157088293651 0.152078609221
    91 0.294142101285 0.182956664147 0.214080901178 0.152805059524 0.146063240111
    92 0.284396573444 0.181753590325 0.214180747568 0.150339781746 0.14582388511
    93 0.277729906778 0.174310909549 0.203307731695 0.146220238095 0.140456034266
    94 0.261934996221 0.167493071303 0.193558627752 0.142242063492 0.135172587554
    95 0.260166288738 0.162047115142 0.187483358935 0.137405753968 0.131403376165
    96 0.246883345931 0.157071050642 0.181280081925 0.133618551587 0.128305618544
    97 0.233373645755 0.15119425548 0.176159754224 0.127008928571 0.122595112119
    98 0.224036281179 0.145715545477 0.167432155658 0.124677579365 0.119029982363
    99 0.220312421265 0.140925925926 0.164152585765 0.118425099206 0.114274376417
    """
  
    lines = data63.split("\n")
    c_lt = []
    c_mx = []
    c_mx_lt = []
    c_mx_pv = []
    c_pv = []
    for line in lines:
      if len(line) > 0:
        ps = line.split()
        if len(ps) == 6:
          c_lt.append(1-float(ps[1]))
          c_mx.append(1-float(ps[2]))
          c_mx_lt.append(1-float(ps[3]))
          c_mx_pv.append(1-float(ps[4]))
          c_pv.append(1-float(ps[5]))
  
    figCDF(c_lt, c_mx, c_mx_lt, c_mx_pv, c_pv,
           "Portion of disconnected pair versus number of failed links", 
           dontSort = True)
    P.show()
  
  
  ########### RECYCLED CODE ##########################333
  #
  #def computeSingleFailurePairs(fbase, num_files, top_file):  
  #  pls = {} # {pid -> pathlet}
  #  fts = {} # {lid -> {fid -> ([push_fids], nh_lid)}}
  #  pllinks = {} # {pid -> set(as_edge_lowest_first)}
  #  plends = {} # {pid -> (first_lid, last_lid)}
  #  
  #  # get all pls and fts)#  for i in range(1, num_files + 1):
  #    f = open("%s_%d.txt" % (fbase, i), "r")
  #    ps, vlist = pickle.load(f)
  #    for pid, pl in ps.iteritems():
  #      if pid not in pls:
  #        pls[pid] = pl
  #        pllinks[pid] = getPlEdges(pl)
  #        plends[pid] = (pl.seq[0], pl.seq[-1])
  #    for lid, ft in vlist:
  #      fts[lid] = ft
  #    f.close()
  #  
  #  # build the graph
  #  g = NX.Graph()
  #  f = open(top_file, "r")
  #  for line in f:
  #    if len(line) == 0 or line[0] == "#":
  #      continue
  #    ps = line.split()
  #    a, b = int(ps[0]), int(ps[1])
  #    g.add_edge(a,b)
  #  f.close()
  #
  #  # set of source vnodes - customer ingress vnodes (aid, 1)
  #  src_set = set(map(lambda x: (x, 1), g.nodes())) 
  #  # set of destination vnodes - (aid, 0)
  #  dst_set = set(map(lambda x: (x, 0), g.nodes())) 
  #  
  #  # fail none and all single edges
  #  all_edges = [(-2,-1)]
  #  all_edges.extend(g.edges())
  #  tot_pairs = 0
  #  max_pairs = -1
  #  for e in all_edges:
  #    assert e[0] < e[1]
  #    
  #    # build a graph from pls that are not affected by the failed link e
  #    fg = NX.DiGraph()
  #    for pid, links in pllinks.iteritems():
  #      if e not in links:
  #        start, end = plends[pid]
  #        fg.add_edge(start, end)
  #  
  #    # compute reachabilities
  #    cntd_pairs = 0
  #    for src in src_set:
  #      #cntd = len(dst_set.intersection(NX.dfs_preorder(fg, src)))
  #      cntd_pairs += len(dst_set.intersection(NX.dfs_preorder(fg, src)))
  #      #if cntd != 10:
  #      #  print src, list(dst_set.difference(NX.dfs_preorder(fg, src)))
  #    
  #    if e != (-2,-1):
  #      tot_pairs += cntd_pairs
  #    else:
  #      max_pairs = cntd_pairs
  #
  #  print tot_pairs, float(tot_pairs) / (max_pairs * (len(all_edges) - 1))
  #
  #
  #def computeFailurePairs(params, num_failues):  
  #  g, fg, pls, fts, pllinks, plends, ase2pids, src_set, dst_set, \
  #      kpids, kgs = params
  #
  #  # fail edges
  #  all_edge_sets = [set()]
  #  all_edge_sets.extend(enum_subsets(g.edges(), num_failues))
  #  tot_pairs = 0
  #  max_pairs = -1
  #  counter = 0
  #  start_clock = time.time()
  #  for fedges in all_edge_sets:
  #    num_edges = len(fg.edges())
  #    counter += 1
  #    if counter % 1000 == 0:
  #      rate = (time.time() - start_clock)/counter
  #      print "%d failures at rate %f s/f, %d more seconds to go" % \
  #          (counter, rate, (len(all_edge_sets) - counter) * rate)
  #                                                                      
  #    # get all the pls that depend on fedges
  #    removed_pids = set()
  #    for e in fedges:
  #      removed_pids.update(ase2pids[e])
  #
  #    # remove these pls from the graph
  #    for pid in removed_pids:
  #      start, end = plends[pid]
  #      if not fg.has_edge(start, end):
  #        print fedge, start, end
  #        pdb.set_trace()
  #      fg.remove_edge(start, end)
  #  
  #    # compute reachabilities
  #    cntd_pairs = 0
  #    for src in src_set:
  #      cntd_pairs += len(dst_set.intersection(NX.dfs_preorder(fg, src)))
  #
  #    # restore the graph
  #    for pid in removed_pids:
  #      start, end = plends[pid]
  #      fg.add_edge(start, end)
  #
  #    if len(fedges) > 0:
  #      tot_pairs += cntd_pairs
  #    else:
  #      max_pairs = cntd_pairs
  #
  #  return (tot_pairs, float(tot_pairs) / (max_pairs * (len(all_edge_sets) - 1)))
  #400 connectivity results for 20 failures per num:
  
  if 0:
    all_ratios_80_5_know = [
      [0.018437499999999996, 0.021718750000000009, 0.021375000000000033, 0.028531250000000008, 0.026312499999999961, 0.024791666666666656, 0.025821428571428551, 0.024289062499999958, 0.02458333333333329, 0.025325000000000042, 0.025284090909090895, 0.024687499999999973, 0.026240384615384582, 0.025816964285714339, 0.025654166666666645, 0.026812500000000017, 0.026708333333333334, 0.025072916666666667, 0.021222222222222253, 0.023074999999999957, 0.024367424242424218, 0.023159722222222179, 0.022522435897435855], 
      [0.15837500000000004, 0.089875000000000038, 0.068020833333333308, 0.057484375000000032, 0.051037499999999958, 0.049656249999999957, 0.047892857142857181, 0.044421875000000055, 0.041534722222222209, 0.039431250000000029, 0.037761363636363621, 0.036505208333333372, 0.034783653846153895, 0.034040178571428603, 0.033475000000000033, 0.032267361111111059, 0.037860119047619101, 0.036541666666666694, 0.035400462962962953, 0.0355375, 0.034155303030303008, 0.034076388888888864, 0.032631410256410254], 
      [0.016437499999999994, 0.081531250000000055, 0.06135416666666671, 0.05209375000000005, 0.045912500000000023, 0.042093750000000041, 0.043660714285714275, 0.054796874999999967, 0.051576388888888935, 0.048118749999999988, 0.045499999999999985, 0.043640624999999988, 0.039879807692307701, 0.039874999999999994, 0.038591666666666691, 0.034968749999999993, 0.034970238095238138, 0.031997395833333386, 0.031807870370370361, 0.034141666666666626, 0.031609848484848491, 0.030661458333333336, 0.029706730769230805], 
      [0.02200000000000002, 0.072437500000000044, 0.061416666666666675, 0.051171874999999978, 0.044137500000000052, 0.041010416666666716, 0.037830357142857096, 0.036421875000000048, 0.035000000000000031, 0.035900000000000043, 0.034965909090909131, 0.033645833333333375, 0.032923076923076944, 0.032933035714285741, 0.032020833333333387, 0.03284722222222225, 0.032720238095238052, 0.032528645833333369, 0.03067824074074077, 0.030197916666666713, 0.033017045454545424, 0.034397569444444498, 0.030102564102564067], 
      [0.025375000000000036, 0.023781249999999976, 0.065666666666666651, 0.062015625000000019, 0.053649999999999975, 0.049499999999999988, 0.046258928571428548, 0.045601562500000026, 0.04397916666666668, 0.042006250000000023, 0.04037500000000005, 0.039744791666666668, 0.048480769230769272, 0.047075892857142865, 0.046945833333333353, 0.04384722222222226, 0.040714285714285703, 0.038666666666666627, 0.038233796296296307, 0.033458333333333368, 0.031280303030303047, 0.027340277777777811, 0.029089743589743589]]
    
    all_ratios_80_10_kno = [[0.028249999999999997, 0.037124999999999964, 0.035749999999999948, 0.052859375000000042, 0.047300000000000009, 0.043666666666666631, 0.046160714285714333, 0.042976562500000037, 0.043145833333333328, 0.046068749999999992, 0.045613636363636356, 0.044026041666666682, 0.047908653846153837, 0.047008928571428577, 0.046858333333333335, 0.051423611111111156, 0.050607142857142851, 0.045940104166666718, 0.036599537037037, 0.040314583333333376, 0.042551136363636388, 0.038843749999999955, 0.037043269230769282], [0.25731250000000006, 0.14634374999999999, 0.11112500000000003, 0.093484374999999953, 0.083875000000000033, 0.085739583333333313, 0.085633928571428597, 0.079312500000000008, 0.073291666666666644, 0.069143749999999948, 0.066045454545454518, 0.063854166666666656, 0.060177884615384647, 0.058781250000000007, 0.057649999999999979, 0.056326388888888856, 0.068214285714285672, 0.065768229166666692, 0.064462962962962944, 0.065952083333333356, 0.063935606060606109, 0.063020833333333304, 0.059943910256410216], [0.022499999999999964, 0.13615624999999998, 0.10114583333333338, 0.088734375000000032, 0.07845000000000002, 0.074489583333333331, 0.080017857142857141, 0.10028906250000003, 0.094798611111111097, 0.087562499999999988, 0.08226704545454544, 0.07914583333333336, 0.070927884615384573, 0.072473214285714294, 0.069745833333333285, 0.06351041666666668, 0.063470238095238107, 0.056317708333333383, 0.055402777777777801, 0.058604166666666679, 0.052721590909090899, 0.051112847222222202, 0.049374999999999947], [0.040874999999999995, 0.1348125, 0.1181875, 0.09757812499999996, 0.0824125, 0.075458333333333294, 0.068937499999999985, 0.066039062500000023, 0.063027777777777794, 0.066374999999999962, 0.064738636363636415, 0.061630208333333325, 0.05994711538461539, 0.060830357142857117, 0.058258333333333301, 0.061124999999999985, 0.060964285714285693, 0.061171874999999987, 0.056738425925925928, 0.055575000000000041, 0.06014204545454549, 0.062458333333333282, 0.052381410256410299], [0.040562499999999946, 0.03862500000000002, 0.10987499999999994, 0.106328125, 0.091812499999999964, 0.085333333333333372, 0.078892857142857098, 0.078906249999999956, 0.076673611111111151, 0.073256250000000023, 0.070232954545454529, 0.06966145833333337, 0.085716346153846157, 0.083526785714285734, 0.083566666666666678, 0.079489583333333336, 0.073122023809523817, 0.069638020833333369, 0.069087962962962934, 0.059497916666666706, 0.054706439393939377, 0.046522569444444439, 0.047815705128205099]]
  
    all_ratios_80_10_all = [[0.015750000000000042, 0.024625000000000008, 0.023249999999999993, 0.040359374999999975, 0.034800000000000053, 0.031166666666666676, 0.033660714285714266, 0.030476562499999971, 0.030645833333333372, 0.033568750000000036, 0.03220454545454543, 0.026874999999999982, 0.031115384615384656, 0.030540178571428545, 0.030654166666666649, 0.034253472222222192, 0.029395833333333288, 0.025679687500000048, 0.019599537037036985, 0.02282083333333329, 0.024587121212121255, 0.022055555555555606, 0.020934294871794878], [0.24587499999999995, 0.13437500000000002, 0.098979166666666618, 0.081250000000000044, 0.071587500000000026, 0.073416666666666686, 0.073285714285714287, 0.06694531250000002, 0.06090972222222224, 0.056749999999999967, 0.053642045454545428, 0.050270833333333376, 0.04477403846153849, 0.043584821428571452, 0.039987499999999954, 0.038663194444444438, 0.050619047619047564, 0.048567708333333348, 0.045361111111111074, 0.043764583333333329, 0.040089015151515195, 0.038276041666666649, 0.036825320512820503], [0.010000000000000009, 0.12393750000000003, 0.084729166666666633, 0.073296875000000039, 0.060937499999999978, 0.057812500000000044, 0.064026785714285661, 0.084804687500000031, 0.075076388888888901, 0.068300000000000027, 0.063619318181818207, 0.058822916666666614, 0.048908653846153838, 0.051151785714285691, 0.048916666666666719, 0.043388888888888921, 0.041910714285714246, 0.037325520833333292, 0.036053240740740788, 0.040437500000000015, 0.035621212121212165, 0.034175347222222263, 0.032637820512820492], [0.028375000000000039, 0.12265625000000002, 0.10593750000000002, 0.085265625000000012, 0.070062500000000028, 0.06308333333333338, 0.054875000000000007, 0.052179687500000016, 0.049312499999999981, 0.047656249999999956, 0.042795454545454525, 0.039187500000000042, 0.03820192307692305, 0.037000000000000033, 0.033725000000000005, 0.038850694444444445, 0.036922619047619065, 0.036914062499999956, 0.035488425925925937, 0.034664583333333332, 0.037562500000000054, 0.036871527777777802, 0.033565705128205114], [0.02806249999999999, 0.026124999999999954, 0.097729166666666645, 0.094093749999999976, 0.079524999999999957, 0.073010416666666633, 0.066544642857142899, 0.066539062499999968, 0.061569444444444454, 0.05650624999999998, 0.049017045454545438, 0.049171874999999976, 0.065908653846153853, 0.064205357142857133, 0.06462083333333335, 0.059545138888888904, 0.051407738095238131, 0.041197916666666612, 0.037888888888888861, 0.032027083333333373, 0.032486742424242432, 0.027131944444444445, 0.029966346153846191]]
  
    all_ratios_80_5_all = [[0.00593750000000004, 0.0092187500000000533, 0.0088749999999999662, 0.016031250000000052, 0.013812500000000005, 0.012291666666666701, 0.013321428571428595, 0.011789062500000003, 0.012083333333333335, 0.012824999999999975, 0.012465909090909055, 0.010385416666666702, 0.012076923076923096, 0.011776785714285753, 0.011716666666666709, 0.012517361111111125, 0.010547619047619028, 0.0093593749999999476, 0.0071458333333332957, 0.0087666666666667004, 0.0098314393939393785, 0.0089253472222222685, 0.0084775641025640613], [0.14662500000000001, 0.077749999999999986, 0.055770833333333325, 0.045171874999999972, 0.038687499999999986, 0.037281250000000044, 0.035499999999999976, 0.032015624999999992, 0.029118055555555578, 0.027006250000000009, 0.025329545454545466, 0.023666666666666614, 0.021355769230769206, 0.020678571428571435, 0.019050000000000011, 0.017937500000000051, 0.023499999999999965, 0.022364583333333354, 0.020650462962962912, 0.019597916666666659, 0.017706439393939344, 0.017295138888888895, 0.016626602564102533], [0.0039375000000000382, 0.069124999999999992, 0.046895833333333359, 0.038124999999999964, 0.031325000000000047, 0.027854166666666624, 0.029714285714285693, 0.041070312500000039, 0.036305555555555591, 0.033074999999999966, 0.030687499999999979, 0.027937499999999948, 0.023629807692307714, 0.02389285714285716, 0.02277499999999999, 0.019743055555555555, 0.019050595238095269, 0.017054687500000054, 0.016712962962962985, 0.019487499999999991, 0.017363636363636359, 0.016446180555555578, 0.015506410256410308], [0.0094999999999999529, 0.060093749999999946, 0.049020833333333291, 0.038749999999999951, 0.03169999999999995, 0.028562500000000046, 0.024866071428571446, 0.02351562500000004, 0.02213194444444444, 0.020768750000000002, 0.018346590909090854, 0.016760416666666722, 0.016293269230769236, 0.015531249999999996, 0.014191666666666714, 0.016079861111111149, 0.015309523809523773, 0.014742187500000004, 0.014351851851851838, 0.014027083333333357, 0.016547348484848512, 0.016161458333333378, 0.014626602564102531], [0.01287499999999997, 0.01128125000000002, 0.053416666666666668, 0.049703124999999959, 0.041300000000000003, 0.037124999999999964, 0.033866071428571454, 0.033195312499999963, 0.030249999999999999, 0.02766875000000002, 0.023869318181818144, 0.023572916666666721, 0.032629807692307722, 0.031428571428571472, 0.031433333333333313, 0.027895833333333342, 0.024193452380952385, 0.018778645833333329, 0.017099537037037038, 0.014822916666666686, 0.014998106060606031, 0.012701388888888943, 0.014868589743589733]]
  
  
    P.figure()
    lt_nums = [1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,18,21,24,27,30,33,36,39]
  
    # get the average of all ratios
    ave_ratios = []
    for ltn in range(0, len(lt_nums)):
      ave_ratios.append(0)
      for run in range(0, 5):
        ave_ratios[-1] += all_ratios_80_10_all[run][ltn]
      ave_ratios[-1] = ave_ratios[-1] / 5.0
    P.plot(lt_nums, ave_ratios, label="Using all pathlets")
  
    ave_ratios = []
    for ltn in range(0, len(lt_nums)):
      ave_ratios.append(0)
      for run in range(0, 5):
        ave_ratios[-1] += all_ratios_80_10_kno[run][ltn]
      ave_ratios[-1] = ave_ratios[-1] / 5.0
    P.plot(lt_nums, ave_ratios, label="Using known pathlets")
  
    P.title("Average of 5 runs using all and known pathlets")
    P.legend()
    P.savefig("/home/igor/pl/data/lt_ratios/80_all_kno_ave_10.png")
    P.show()
    
  
  if 0:
    data400_all = [
    (10, 0.9846635186045311, 0.9697211900707023, 0.9777225274010049, 0.96186024503158801, 0.9625025803787996),
    (20, 0.9734685451824327, 0.9409689967487227, 0.9585821769805524, 0.92366485919635777, 0.92349950972802808),
    (30, 0.9555491046085565, 0.9097731847035144, 0.9324542448777694, 0.88749008875361279, 0.88265082314083709),
    (40, 0.9407970790111988, 0.8826575966351860, 0.9080823201687017, 0.85767897792669512, 0.85352931310316349),
    (50, 0.92440199721319094, 0.85216235743407132, 0.89007706125849362, 0.81491290891884283, 0.81355311709758993),
    (60, 0.90826430820044379, 0.82525803787996077, 0.85760459243445886, 0.79347903931247921, 0.78577211384631263),
    (70, 0.89591397017082108, 0.80203914434639001, 0.84420348337715756, 0.760614625163056, 0.75375380605872944),
    (80, 0.87794872787325184, 0.77199256850905718, 0.81680573273281087, 0.72796570069314781, 0.71993310367962016),
    (90, 0.86370568199411679, 0.74577527481034211, 0.79366457525188094, 0.69872624498043323, 0.69278526087629666),
    (100, 0.83864246271352638, 0.7186929091190587, 0.76909283798911765, 0.66917730260633812, 0.66251193425194821),
    (110, 0.80846493265211328, 0.68407680497497036, 0.73887232823930649, 0.63024272962119854, 0.62481098725292872),
    (120, 0.80753341590545491, 0.66584997677659086, 0.72361953607039653, 0.60909404813668566, 0.60351544356711562),
    (130, 0.79200921195231455, 0.64470538525055476, 0.70395915232615658, 0.58649129089188434, 0.57791163492800746),
    (140, 0.76750787015533883, 0.61673633689425611, 0.67891163989482184, 0.55565196818170193, 0.55252780358156572),
    (150, 0.73879535015740305, 0.58942786551065696, 0.64747077660045305, 0.53240338133360621, 0.52567670434019709),
    (160, 0.73010915002322341, 0.56791589255302677, 0.62816055296659812, 0.50872829117323581, 0.50216816328637048),
    (170, 0.71558677813903082, 0.54445605614904269, 0.60940160370727137, 0.48065004987594956, 0.47565348093100068),
    (180, 0.69299943231666405, 0.52800872168034263, 0.59236872250136685, 0.46477798808092691, 0.45853589306910253),
    (190, 0.66520681736078857, 0.50035673736904573, 0.55830100752388634, 0.44342916336291788, 0.43430839397223514),
    (200, 0.62500967642049854, 0.45811496877741653, 0.51549491031215011, 0.40174182162314243, 0.39516856324508437)]
    
    data400_kno = [
    (10, 0.98354492439490115, 0.9685600196108789, 0.97511520137460628, 0.96211985574340742, 0.96317283377199769), 
    (20, 0.97184419672807965, 0.94382063786963921, 0.95893103538048996, 0.92897536895413968, 0.92703978944109), 
    (30, 0.95965706765753211, 0.90748116323476291, 0.93371560230142403, 0.88170703634549963, 0.88330946482943695), 
    (40, 0.93842571089435933, 0.87741652474583265, 0.90852750514175629, 0.84685142082512721, 0.84394643133612013),
    (50, 0.92221706146462301, 0.85052446199102028, 0.87634401603707268, 0.82515794050694424, 0.81684019714094025), 
    (60, 0.91261547195128245, 0.82892282087010372, 0.86209679518887816, 0.79633092053098709, 0.7921666150590907),
    (70, 0.88909918976105695, 0.7932148939464313, 0.8271224388846945, 0.75990229429367984, 0.75062316148010533), 
    (80, 0.88144836662021986, 0.77369884399029776, 0.82038608731873686, 0.72783077985523181, 0.72134295814625582), 
    (90, 0.86419595396604221, 0.74793376167621406, 0.79229127072973893, 0.70435455405785607, 0.69250593487123913),
    (100, 0.84057258605563301, 0.70901132786293031, 0.75579000807060481, 0.66305343120955573, 0.65156693502606178),
    (110, 0.83266630541363473, 0.69118671620993966, 0.74734060555570014, 0.63601810880630227, 0.63392682045724313),
    (120, 0.80733472673788509, 0.67372980853589304, 0.72570357449688894, 0.62266797964038156, 0.61802072044176082),
    (130, 0.78186638798575636, 0.63295659802859061, 0.68470360053109791, 0.58211755377650454, 0.57172162873509835), 
    (140, 0.7506683181090984, 0.59996258450740569, 0.65893754393272763, 0.54202240581118755, 0.54014875883779734), 
    (150, 0.74803762192289824, 0.59430832946276513, 0.65192132462055141, 0.53770621786837869, 0.53007173453062906), 
    (160, 0.72451230840687408, 0.56033377199772927, 0.62044336257842803, 0.50127887050157305, 0.49836016927284926), 
    (170, 0.72066238323785936, 0.54621780977447487, 0.60190830751607616, 0.49150446325805047, 0.47796356505134951), 
    (180, 0.69449347164163699, 0.52085204107963046, 0.58267553565384911, 0.46011330792643934, 0.45717345306290963), 
    (190, 0.67603473189864272, 0.49771313928884758, 0.56019369451459222, 0.43632887433818451, 0.43178123548536923), 
    (200, 0.64615136502038495, 0.48526087629664033, 0.53612767176069354, 0.43528659487940252, 0.42528061619445734)]
  
    c_lt = []
    c_mx = []
    c_mx_lt = []
    c_mx_pv = []
    c_pv = []
    for i in range(0, 20):
      c_lt.append(1 - data400_all[i][1])
      c_mx.append(1 - data400_all[i][2])
      c_mx_lt.append(1 - data400_all[i][3])
      c_mx_pv.append(1 - data400_all[i][4])
      c_pv.append(1 - data400_all[i][5])
  
    figCDF(c_lt, c_mx, c_mx_lt, c_mx_pv, c_pv,
           "Portion of disconnected pair versus number of failed links (using all pathlets))", 
           dontSort = True, xaxis = range(10, 201, 10))
  
    c_kno_lt = []
    c_kno_mx = []
    c_kno_mx_lt = []
    c_kno_mx_pv = []
    c_kno_pv = []
    for i in range(0, 20):
      c_kno_lt.append(1 - data400_kno[i][1])
      c_kno_mx.append(1 - data400_kno[i][2])
      c_kno_mx_lt.append(1 - data400_kno[i][3])
      c_kno_mx_pv.append(1 - data400_kno[i][4])
      c_kno_pv.append(1 - data400_kno[i][5])
  
    figCDF(c_kno_lt, c_kno_mx, c_kno_mx_lt, c_kno_mx_pv, c_kno_pv,
           "Portion of disconnected pair versus number of failed links (using known pathlets))", 
           dontSort = True, xaxis = range(10, 201, 10))
  
  
    figCDF6(c_lt, c_mx, c_pv, c_kno_lt, c_kno_mx, c_kno_pv, 
            "Comparison of connectivity when all pathlets are used versus only locally known", 
            xaxis = range(10, 201, 10))
  
    P.show()
