import networkx as NX
import random
import time
import numpy
import pickle
import sys
import pdb
import copy
import math


class PolyGraph:

    OFFSET = 1000000

    @classmethod
    def from_file(cls, fname = "", renumber = False):
        # this assumes that the file is has the following format
        # ASN1 ASN2 relationship
        # relationship = 0 means AS1 and AS2 are peers
        # relationship = 1 means AS1 is a provider of AS2
        # relationship = -1 means AS1 is a customer of AS2
        #
        # There can be multiple entries for the same pair of ASes
        #
        # The internal graph renumbers the ASes to be in range 1:n
        # and we also keep the dictionary of this mapping.
        # These two dictionaries are the only two structures that
        # know about the file indices. All other structs use internal
        # indices
        
        G = NX.Graph()
        
        # maps file indices into internal indices (and back)
        renum = {}
        irenum = {}

        # maps edges (2-tuples of nodes) to policies
        edges = {}
        
        f=open(fname, 'r')
        renumind = 1
        for line in f:
            l = line.split()

            p = int(l[0])
            q = int(l[1])
            policy = int(l[2])
            if renumber:
                if renum.has_key(p):
                    u = renum[p]
                else:
                    renum[p] = renumind
                    irenum[renumind] = p
                    u = renumind
                    renumind += 1
                
                
                if renum.has_key(q):
                    v = renum[q]
                else:
                    renum[q] = renumind
                    irenum[renumind] = q
                    v = renumind
                    renumind += 1
            else:
                renum[p] = p
                renum[q] = q
                irenum[p] = p
                irenum[q] = q
                u = p
                v = q

            if policy == -1:
                policy = 1
                tmp = u
                u = v
                v = tmp

            G.add_edge(u,v)
            #pdb.set_trace()
            if policy == 1:
                # u is a provider of v
                edges[(u,v)] = 1
                edges[(v,u)] = -1
            elif policy == 0:
                edges[(u,v)] = 0
                edges[(v,u)] = 0

        return cls.from_g_and_edges(G, edges)
    

    @classmethod
    def from_g_and_edges(cls, G, edges):
        # maps indices to lists of indices
        nproviders = {}
        ncustomers = {}
        npeers = {}
        nPG = NX.DiGraph()
        
        for e in G.edges():
            u = e[0]
            v = e[1]
            if not nproviders.has_key(u):
                nproviders[u] = []
            if not ncustomers.has_key(u):    
                ncustomers[u] = []
            if not npeers.has_key(u):    
                npeers[u] = []
            if not nproviders.has_key(v):
                nproviders[v] = []
            if not ncustomers.has_key(v):    
                ncustomers[v] = []
            if not npeers.has_key(v):    
                npeers[v] = []

            policy = edges[e]    
            if policy == -1:
                policy = 1
                tmp = u
                u = v
                v = tmp

            if policy == 1:
                # u is a provider of v
                nproviders[v].append(u)
                ncustomers[u].append(v)
            elif policy == 0:
                npeers[v].append(u)
                npeers[u].append(v)

        
        for u in G.nodes():
            for v in ncustomers[u]:
                nPG.add_edge(u, v+PolyGraph.OFFSET)
                nPG.add_edge(u+PolyGraph.OFFSET, v+PolyGraph.OFFSET)
                nPG.add_edge(v, u)
            for v in npeers[u]:
                # we add same edges multiple times here, but it is ok
                nPG.add_edge(u, v+PolyGraph.OFFSET)
                nPG.add_edge(v, u+PolyGraph.OFFSET)

        return cls(G, nPG, nproviders, ncustomers, npeers, edges)


    def __init__(self, G, PG, providers, customers, peers,
                 edges, renum={}, irenum={}):
        self.G = G
        self.PG = PG
        self.providers = providers
        self.customers = customers
        self.peers = peers
        self.edges = edges
        self.renum = renum
        self.irenum = irenum

    def deepcopy(self):
        return PolyGraph(self.G.copy(), self.PG.copy(),
                         copy.deepcopy(self.providers),
                         copy.deepcopy(self.customers),
                         copy.deepcopy(self.peers),
                         copy.deepcopy(self.edges),
                         copy.deepcopy(self.renum),
                         copy.deepcopy(self.irenum))

    def remove_edge(self, u, v):
        self.G.delete_edge(u, v)
        policy = self.edges[(u,v)]
        del self.edges[(u,v)]
        del self.edges[(v,u)]
        
        if policy == -1:
            policy = 1
            tmp = u
            u = v
            v = tmp

        if policy == 1:
            self.PG.delete_edge(v,u)
            self.PG.delete_edge(u,v+PolyGraph.OFFSET)
            self.PG.delete_edge(u+PolyGraph.OFFSET,
                                v+PolyGraph.OFFSET)
            self.providers[v].remove(u)
            self.customers[u].remove(v)
        elif policy == 0:
            self.PG.delete_edge(u,v+PolyGraph.OFFSET)
            self.PG.delete_edge(v,u+PolyGraph.OFFSET)
            self.peers[v].remove(u)
            self.peers[u].remove(v)
        else:
            sys.stderr.write("Invalid policy")
            sys.exit(28)
 
    def add_edge(self, u, v, policy):
        if not nproviders.has_key(u):
            nproviders[u] = []
        if not ncustomers.has_key(u):    
            ncustomers[u] = []
        if not npeers.has_key(u):    
            npeers[u] = []
        if not nproviders.has_key(v):
            nproviders[v] = []
        if not ncustomers.has_key(v):    
            ncustomers[v] = []
        if not npeers.has_key(v):    
            npeers[v] = []

        policy = edges[e]    
        if policy == -1:
            policy = 1
            tmp = u
            u = v
            v = tmp

        if policy == 1:
            # u is a provider of v
            nproviders[v].append(u)
            ncustomers[u].append(v)
        elif policy == 0:
            npeers[v].append(u)
            npeers[u].append(v)

        
        for u in G.nodes():
            for v in ncustomers[u]:
                nPG.add_edge(u, v+PolyGraph.OFFSET)
                nPG.add_edge(u+PolyGraph.OFFSET, v+PolyGraph.OFFSET)
                nPG.add_edge(v, u)
            for v in npeers[u]:
                # we add same edges multiple times here, but it is ok
                nPG.add_edge(u, v+PolyGraph.OFFSET)
                nPG.add_edge(v, u+PolyGraph.OFFSET)
       

    def ssspathl(self, s):
        #costs = NX.single_source_dijkstra_path_length(self.PG, s)
        costs = NX.single_source_shortest_path_length(self.PG,s)
        uc = {}
        for k in self.G.nodes():
            if costs.has_key(k) and costs.has_key(k+PolyGraph.OFFSET):
                uc[k] = min(costs[k], costs[k+PolyGraph.OFFSET])
            elif costs.has_key(k+PolyGraph.OFFSET):    
                uc[k] = costs[k+PolyGraph.OFFSET]
            elif costs.has_key(k):    
                uc[k] = costs[k]

        return uc

    def spath(self,u,v, node_path = False):
        # returns a path as a list of edges or False if there is
        # no policy compliant path
        # links are always a tuple where the smaller
        # index is the first, (small_ind, large_ind), for easy comparison
        path1 = NX.bidirectional_shortest_path(self.PG,u,v)
        path2 = NX.bidirectional_shortest_path(self.PG,u,v+PolyGraph.OFFSET)

        if path1 == False and path2 == False:
            return False
        elif path1 == False:
            path = path2
        elif path2 == False:
            path = path1
        elif len(path1) < len(path2):
            path = path1
        else:
            path = path2
            
        for i in range(0,len(path)):
            path[i]= path[i] % PolyGraph.OFFSET

        lp = []
        for i in range(0,len(path)-1):
            if path[i] < path[i+1]:
                lp.append((path[i], path[i+1]))
            else:
                lp.append((path[i+1], path[i]))

        if node_path:
            return (lp, path)
        else:
            return lp
        
    def get_pos(self):
        n = self.G.order()
        m = float(n)
        pos = {}
        ind = 1
        for i in self.G.nodes():
            x = ind/m
            pos[i] = numpy.array([x, 0.4 - pow((m+1)/(2*m) - x,2)])
            pos[i+PolyGraph.OFFSET] = numpy.array( \
                [x, -0.4 + pow((m+1)/(2*m) - x,2)])
            ind += 1
        return pos

    def delta_subgraph(self, src, dst, maxn):
        # returns a new PolyGraph traversed by amr with delta = maxn
        # src and dst should be regular indices, i.e. no more than n
        n = self.G.order()

        # maps steps into sets of nodes reachable in these many steps.
        ofields = {}
        ofields[-1] = set()
        ofields[0] = set([src])
        diffs = {}
        diffs[0] = ofields[0] - ofields[-1]
        for step in range(1, maxn):
            s = set()
            newnodes = diffs[step-1]

            for r in newnodes:
                oes = self.PG.out_edges(r)
                for e in oes:
                    s.add(e[1])


            ofields[step] = s | ofields[step-1]
            diffs[step] = ofields[step] - ofields[step-1]


        ifields = {}
        ifields[-1] = set()
        ifields[0] = set([dst, dst+PolyGraph.OFFSET])
        for step in range(1, maxn):
            #pdb.set_trace()
            pps = ifields[step-2]
            ps = ifields[step-1]
            s = set()
            
            newnodes = ps - pps
            for r in newnodes:
                ies = self.PG.in_edges(r)
                for e in ies:
                    s.add(e[0])

            ifields[step] = s | ps


        nG = NX.Graph()
        nedges = {}
        for step in range(0,maxn):
            #pdb.set_trace()
            for r in diffs[step]:
                for e in self.PG.out_edges(r):
                    if e[1] in ifields[maxn-step-1]:
                        u = e[0] % PolyGraph.OFFSET
                        v = e[1] % PolyGraph.OFFSET
                        nG.add_edge(u, v)
                        nedges[(u,v)] = self.edges[(u,v)]
                        nedges[(v,u)] = self.edges[(v,u)]
        
        return PolyGraph.from_g_and_edges(nG, nedges)

    def upgraph(self, u):
        # return a new NX.Graph of recursive providers of u
        G = NX.Graph()
        discov = set([u])
        expanded = set()

        while len(discov) != 0:
            n = discov.pop()
            expanded.add(n)
            for p in self.providers[n]:
                G.add_edge(n,p)
                if p not in discov and p not in expanded:
                    discov.add(p)
                    
                if p in discov and p in expanded:
                    sys.stderr.write("upgragh is buggy")
                    sys.exit(26)

        return G

    def downgraph(self, u):
        # return a new NX.Graph of recursive providers of u
        G = NX.Graph()
        discov = set([u])
        expanded = set()

        while len(discov) != 0:
            n = discov.pop()
            expanded.add(n)
            for p in self.customers[n]:
                G.add_edge(n,p)
                if p not in discov and p not in expanded:
                    discov.add(p)
                    
                if p in discov and p in expanded:
                    sys.stderr.write("downgragh is buggy")
                    sys.exit(25)

        return G
        
        
    def __str__(self):
        return "a"
